#include "ATen/cuda/CUDAContext.h"
#include "torch/script.h"
#include <cub/cub.cuh>

#define PRINT(x) std::cerr << #x " = " << x << "\n";

void bla(){ 
    auto stream = at::cuda::getCurrentCUDAStream();
    auto cuda_device_props = at::cuda::getCurrentDeviceProperties();
    const int texture_alignment = cuda_device_props->textureAlignment;

    PRINT(stream)
    PRINT(cuda_device_props)
    PRINT(texture_alignment)

    // Declare, allocate, and initialize device-accessible pointers for sorting data
    int  num_items = 123456;          // e.g., 7
    int  *d_keys_in = nullptr;         // e.g., [8, 6, 7, 5, 3, 0, 9]
    int  *d_keys_out = nullptr;        // e.g., [        ...        ]
    int  *d_values_in = nullptr;       // e.g., [0, 1, 2, 3, 4, 5, 6]
    int  *d_values_out = nullptr;      // e.g., [        ...        ]

    if(true)
    {
        // allocation is actually not necessary for determining the temp size
        cudaMalloc(&d_keys_in, num_items*sizeof(int));
        cudaMalloc(&d_keys_out, num_items*sizeof(int));
        cudaMalloc(&d_values_in, num_items*sizeof(int));
        cudaMalloc(&d_values_out, num_items*sizeof(int));
        PRINT(d_keys_in)
        PRINT(d_keys_out)
        PRINT(d_values_in)
        PRINT(d_values_out)
    }
    // Determine temporary device storage requirements
    void     *d_temp_storage = NULL;
    size_t   temp_storage_bytes = 0;
    cub::DeviceRadixSort::SortPairs(d_temp_storage, temp_storage_bytes,
        d_keys_in, d_keys_out, d_values_in, d_values_out, num_items);

    PRINT(d_temp_storage)
    PRINT(temp_storage_bytes)
    if(temp_storage_bytes == 0){
        std::cerr << "temp_storage_bytes should not be 0!\n";
        std::cerr << "temp_storage_bytes should not be 0!\n";
        std::cerr << "temp_storage_bytes should not be 0!\n";
    }

    // Allocate temporary storage
    if(temp_storage_bytes){
        cudaMalloc(&d_temp_storage, temp_storage_bytes);
        // Run sorting operation
        cub::DeviceRadixSort::SortPairs(d_temp_storage, temp_storage_bytes,
            d_keys_in, d_keys_out, d_values_in, d_values_out, num_items);
        cudaFree(d_temp_storage);
    }

    cudaFree(d_keys_in);
    cudaFree(d_keys_out);
    cudaFree(d_values_in);
    cudaFree(d_values_out);

}